@@ -7,15 +7,15 @@ import socket
 from ConfigParser import RawConfigParser
 import logging
 from logging.handlers import SysLogHandler
-import gc
 
 from optparse import OptionParser
-from pyftpdlib import ftpserver
+import pyftpdlib.servers
 
 from server import RackspaceCloudFilesFS
 from constants import version, default_address, default_port, \
-    default_config_file, default_banner, default_workers
-from monkeypatching import MyFTPHandler, MyDTPHandler
+    default_config_file, default_banner, \
+    default_ks_tenant_separator, default_ks_service_type, default_ks_endpoint_type
+from monkeypatching import MyFTPHandler
 from multiprocessing import Manager
 
 def modify_supported_ftp_commands():
@@ -24,37 +24,23 @@ def modify_supported_ftp_commands():
         'SITE CHMOD',
     )
     for cmd in unsupported:
-        if cmd in ftpserver.proto_cmds:
-            del ftpserver.proto_cmds[cmd]
+        if cmd in pyftpdlib.handlers.proto_cmds:
+            del pyftpdlib.handlers.proto_cmds[cmd]
     # add the MD5 command, FTP extension according to IETF Draft:
     # http://tools.ietf.org/html/draft-twine-ftpmd5-00
-    ftpserver.proto_cmds.update({
+    pyftpdlib.handlers.proto_cmds.update({
         'MD5': dict(perm=None,
                     auth=True,
                     arg=True,
-                    help='Syntax: MD5 <SP> file-name (get MD5 of file)')
+                    help=u'Syntax: MD5 <SP> file-name (get MD5 of file)')
         })
 
-def start_garbage_collector(interval=10):
-    """Starts the garbage collector at the interval in seconds. 0 means
-    disabled"""
-    def garbage_collect():
-        """
-        Run the garbage collector every interval seconds to make sure
-        sleeping daemons get cleaned properly
-        """
-        ftpserver.CallLater(interval, garbage_collect)
-        gc.collect()
-    if interval:
-        garbage_collect()
-
 class Main(object):
     """ FTPCloudFS: A FTP Proxy Interface to Rackspace Cloud Files or
     OpenStack swift."""
 
     def __init__(self):
         self.options = None
-        self._workers = []
 
     def setup_log(self):
         ''' Setup Logging '''
@@ -63,9 +49,6 @@ class Main(object):
             """
             Dummy function.
             """
-        ftpserver.log = logging.info
-        ftpserver.logline = logging.debug
-        ftpserver.logerror = logging.error
 
         if self.options.log_level:
             self.options.log_level = logging.DEBUG
@@ -92,12 +75,16 @@ class Main(object):
                                 format=log_format,
                                 level=self.options.log_level)
 
+        # warnings
+        if self.config.get("ftpcloudfs", "workers") is not None:
+            logging.warning("workers configuration token has been deprecated and has no effect")
+
     def parse_configuration(self, config_file=default_config_file):
         ''' Parse Configuration File '''
         config = RawConfigParser({'banner': default_banner,
                                   'port': default_port,
                                   'bind-address': default_address,
-                                  'workers': default_workers,
+                                  'workers': None,
                                   'memcache': None,
                                   'max-cons-per-ip': '0',
                                   'auth-url': None,
@@ -109,6 +96,12 @@ class Main(object):
                                   'uid': None,
                                   'gid': None,
                                   'masquerade-firewall': None,
+                                  # keystone auth 2.0 support
+                                  'keystone-auth': False,
+                                  'keystone-region-name': None,
+                                  'keystone-tenant-separator': default_ks_tenant_separator,
+                                  'keystone-service-type': default_ks_service_type,
+                                  'keystone-endpoint-type': default_ks_endpoint_type,
                                  })
         config.read(default_config_file)
         if not config.has_section('ftpcloudfs'):
@@ -133,13 +126,6 @@ class Main(object):
                           help="Address to bind by default: %s." % \
                               (default_address))
 
-        parser.add_option('--workers',
-                          type="int",
-                          dest="workers",
-                          default=self.config.get('ftpcloudfs', 'workers'),
-                          help="Number of workers to use default: %d." % \
-                              (default_workers))
-
         memcache = self.config.get('ftpcloudfs', 'memcache')
         if memcache:
             memcache = [x.strip() for x in memcache.split(',')]
@@ -209,20 +195,55 @@ class Main(object):
                           help="GID to drop the privilige to " + \
                               "when in daemon mode.")
 
+        parser.add_option('--keystone-auth',
+                          action="store_true",
+                          dest="keystone",
+                          default=self.config.get('ftpcloudfs', 'keystone-auth'),
+                          help="Use auth 2.0 (Keystone, requires keystoneclient).")
+
+        parser.add_option('--keystone-region-name',
+                          type="str",
+                          dest="region_name",
+                          default=self.config.get('ftpcloudfs', 'keystone-region-name'),
+                          help="Region name to be used in auth 2.0.")
+
+        parser.add_option('--keystone-tenant-separator',
+                          type="str",
+                          dest="tenant_separator",
+                          default=self.config.get('ftpcloudfs', 'keystone-tenant-separator'),
+                          help="Character used to separate tenant_name/username in auth 2.0, " + \
+                              "default: TENANT%sUSERNAME." % default_ks_tenant_separator)
+
+        parser.add_option('--keystone-service-type',
+                          type="str",
+                          dest="service_type",
+                          default=self.config.get('ftpcloudfs', 'keystone-service-type'),
+                          help="Service type to be used in auth 2.0, default: %s." % default_ks_service_type)
+
+        parser.add_option('--keystone-endpoint-type',
+                          type="str",
+                          dest="endpoint_type",
+                          default=self.config.get('ftpcloudfs', 'keystone-endpoint-type'),
+                          help="Endpoint type to be used in auth 2.0, default: %s." % default_ks_endpoint_type)
+
         (options, _) = parser.parse_args()
+
+        if options.keystone:
+            keystone_keys = ('region_name', 'tenant_separator', 'service_type', 'endpoint_type')
+            options.keystone = dict((key, getattr(options, key)) for key in keystone_keys)
+
         self.options = options
 
     def setup_server(self):
         """Run the main ftp server loop"""
         banner = self.config.get('ftpcloudfs', 'banner').replace('%v', version)
-        banner = banner.replace('%f', ftpserver.__ver__)
+        banner = banner.replace('%f', pyftpdlib.__ver__)
 
         MyFTPHandler.banner = banner
         RackspaceCloudFilesFS.servicenet = self.options.servicenet
         RackspaceCloudFilesFS.authurl = self.options.authurl
+        RackspaceCloudFilesFS.keystone = self.options.keystone
         RackspaceCloudFilesFS.memcache_hosts = self.options.memcache
-        if self.options.workers > 1 and not self.options.memcache:
-            RackspaceCloudFilesFS.single_cache = False
 
         masquerade = self.config.get('ftpcloudfs', 'masquerade-firewall')
         if masquerade:
@@ -236,9 +257,10 @@ class Main(object):
         except ValueError, errmsg:
             sys.exit('Max connections per IP error: %s' % errmsg)
 
-        ftpd = ftpserver.FTPServer((self.options.bind_address,
-                                    self.options.port),
-                                   MyFTPHandler)
+        ftpd = pyftpdlib.servers.MultiprocessFTPServer((self.options.bind_address,
+                                                        self.options.port),
+                                                       MyFTPHandler,
+                                                       )
 
         # set it to unlimited, we use our own checks with a shared dict
         ftpd.max_cons_per_ip = 0
@@ -272,14 +294,9 @@ class Main(object):
 
     def signal_handler(self, signal, frame):
         """ Catch signals and propagate them to child processes """
-        for pid in self._workers:
-            try:
-                os.kill(pid, signal)
-            except:
-                pass
-
         if self.shm_manager:
             self.shm_manager.shutdown()
+            self.shm_manager = None
         self.old_signal_handler(signal, frame)
 
     def main(self):
@@ -297,8 +314,7 @@ class Main(object):
             ftpd.serve_forever()
             return
 
-        start_garbage_collector()
-        daemonContext = self.setup_daemon([ftpd.socket.fileno(),])
+        daemonContext = self.setup_daemon([ftpd.socket.fileno(), ftpd.ioloop.fileno(),])
         with daemonContext:
             self.old_signal_handler = signal.signal(signal.SIGTERM, self.signal_handler)
 
@@ -306,13 +322,6 @@ class Main(object):
             MyFTPHandler.shared_ip_map = self.shm_manager.dict()
             MyFTPHandler.shared_lock = self.shm_manager.Lock()
 
-            for i in range(self.options.workers):
-                pid = os.fork()
-                if pid == 0:
-                    self.pid = os.getpid()
-                    self.pidfile.close()
-                    self.shm_manager = None
-                    break
-                self._workers.append(pid)
             self.setup_log()
             ftpd.serve_forever()
+
