@@ -1,3 +1,5 @@
+import sys
+import socket
 from pyftpdlib.handlers import DTPHandler, FTPHandler, _strerror
 from ftpcloudfs.utils import smart_str
 from server import RackspaceCloudAuthorizer
@@ -39,7 +41,8 @@ class MyFTPHandler(FTPHandler):
     def process_command(self, cmd, *args, **kwargs):
         '''Flush the FS cache with every new FTP command'''
         if self.fs:
-            self.fs.flush()
+            if self.fs.memcache_hosts is None:
+                self.fs.flush()
             self.fs.connection.real_ip = self.remote_ip
         FTPHandler.process_command(self, cmd, *args, **kwargs)
 
@@ -76,6 +79,17 @@ class MyFTPHandler(FTPHandler):
 
         FTPHandler.handle(self)
 
+    def handle_error(self):
+        """Catch some 'expected' exceptions not processed by FTPHandler/AsyncChat"""
+        # this is aesthetic only
+        t, v, _ = sys.exc_info()
+        if t == socket.error:
+            self.log("Connection error: %s" % v)
+            self.handle_close()
+            return
+
+        FTPHandler.handle_error(self)
+
     def close(self):
         """Remove the ip from the shared map before calling close"""
         if not self._closed and self.max_cons_per_ip and self.shared_ip_map != None:
